///////
     //
     //  MOBOTIX EventStream Client SDK 
     //
     //  See LICENSE_SDK included in the SDK package
     //
     //  Copyright (c) 2005 - 2016, MOBOTIX AG
     //  All rights reserved.
     //
///////
    

#ifndef M1IMAGE_JSIMAGE_SRC_MXPEG_AUDIO_H_
#define M1IMAGE_JSIMAGE_SRC_MXPEG_AUDIO_H_

#include "MxPEG_Defines.hpp"

#include <time.h>

namespace ie { namespace MxPEG
{

typedef enum MxPEG_AudioPacketType_T {
   eapt_Start           = 0,    //MXA   - start of audio stream, contains stream format information
   eapt_Data            = 1,    //MXS   - playload data
   eapt_Stop            = 2,    //MXO   - end of stream
   eapt_NoData          = 3     //no valid audio data
} MxPEG_AudioPacketType;

typedef enum MxPEG_AudioCodec_T {
   eac_pcm              = 0,    //PCM16 audio data
   eac_alaw             = 1,    //alaw audio data
   eac_unsupported      = 2
} MxPEG_AudioCodec;

typedef enum MxPEG_SampleFormat_t {
  easf_U8       = 0,
  easf_S8       = 1,
  easf_S16      = 2,
  easf_U16      = 3,
  easf_S32      = 4,
  easf_F32      = 5,
  easf_unsupported =6
} MxPEG_SampleFormat;


/****************************************************************
 *
 * MxPEG_AudioFormat
 *
 ****************************************************************/
class MxPEG_AudioFormat
{

public:
   MxPEG_AudioFormat(MxPEG_AudioCodec codec, MxPEG_SampleFormat sampleFormat, uint32_t sampleFreq, uint32_t channels);

   MxPEG_AudioCodec codec();
   uint32_t samplefreq();
   MxPEG_SampleFormat sampleFormat();
   uint32_t sampleDuration();
   uint32_t channels();
   uint8_t bytesPerSample();

   inline bool operator==( const MxPEG_AudioFormat& other) {
      return (
            (audioSamplefreq == other.audioSamplefreq) &&
            (audioSampleFormat == other.audioSampleFormat) &&
            (audioCodec == other.audioCodec) &&
            (audioChannels == other.audioChannels)
            );
   };

private:
   MxPEG_AudioCodec audioCodec;
   uint32_t audioSamplefreq;
   MxPEG_SampleFormat audioSampleFormat;
   uint32_t audioChannels;
};

/****************************************************************
 *
 * MxPEG_Audio
 *
 * Interface for audio container class
 *
 ****************************************************************/
// cppcheck-suppress noCopyConstructor
class MxPEG_Audio
{
public:

   /*
    * Returns pointer to audio payload data stored in this container
    */
   virtual
   uint8_t * audioBuffer() = 0;

   /*
    * Size of audio payload data stored in this container
    */
   virtual
   size_t audioBufferSize()= 0;

   virtual
   uint8_t * rawBuffer() = 0;

   virtual
   size_t rawBufferSize() = 0;

   virtual
   MxPEG_Audio * clone() = 0;
   
   struct timeval timeStamp();
   MxPEG_AudioPacketType packetType();
   MxPEG_AudioFormat *format();

   virtual
   ~MxPEG_Audio ();

private:
//   MxPEG_ReturnCode parseAudioData(uint8_t *rawAudio, size_t rawAudioLength);

protected:
   MxPEG_Audio();

protected:
   MxPEG_AudioFormat *mAudioFormat;
   struct timeval mTimeStamp;
   MxPEG_AudioPacketType mAudioType;
   bool mDataOwner;
};

}} /* namespace ie::MxPEG */

#endif /* M1IMAGE_JSIMAGE_SRC_MXPEG_AUDIO_H_ */
