///////
     //
     //  MOBOTIX EventStream Client SDK 
     //
     //  See LICENSE_SDK included in the SDK package
     //
     //  Copyright (c) 2005 - 2016, MOBOTIX AG
     //  All rights reserved.
     //
///////
    

#ifndef MXPEG_SRC_MXPEG_IMAGE_HPP_
#define MXPEG_SRC_MXPEG_IMAGE_HPP_

#include "MxPEG_Defines.hpp"

#include <time.h>
#include <map>

typedef enum t_MxPEG_ImageMode {
   im_RGB = 0,				//RGB, using channel oder bgra interleaved
   im_YUV,				//YUV420 picture data
   im_YUVmx				//MX internal YUV format, uses 4 byte / pixel (like RGBa), stores RGBa.r=Y,RGBa.g=Cr,RGBa.b=Cb => shader needs load only one texture
} MxPEG_ImageMode;

namespace ie { namespace MxPEG
{

   // cppcheck-suppress noCopyConstructor
   class MxPEG_Image {

   public:
      MxPEG_Image(uint32_t width = 0, uint32_t height = 0, MxPEG_ImageMode mode = im_YUV);
      MxPEG_Image(uint8_t* buffer, size_t bufferSize);
      MxPEG_Image(MxPEG_Image *copy);

      virtual
      ~MxPEG_Image();

      uint32_t width();
      uint32_t height();
      bool setSize(uint32_t width, uint32_t height);

      void getImgBuffer(uint8_t** buffer, size_t *bufferSize);
      size_t imgBufferSize();
      uint8_t*imgBuffer();


      //void extractRawBuffer(uint8_t** buffer, size_t *bufferSize);
      void getRawBuffer(uint8_t** buffer, size_t *bufferSize);

      void getRawHeader(uint8_t ** rawHeader, uint32_t * rawHeaderSize);
      void addRawHeader(uint8_t * rawHeader, uint32_t rawHeaderSize);
      void clearRawHeader();

      void setCreationTime(struct timeval creation);
      struct timeval creationTime();
      void setRecordTime(struct timeval record);
      struct timeval recordTime();

      MxPEG_ImageMode mode();

   private:
      void updatePrefix();
      size_t calcBufferSize();

      void updateRawHeader(uint8_t * rawHeader, uint32_t rawHeaderSize);

      bool updateInfo();

   private:
      inline uint32_t readU32At(uint8_t * buffer, size_t pos) {
         return buffer[pos] + (buffer[pos+1] << 8) + (buffer[pos+2] << 16) + (buffer[pos+3] << 24);
      }

      inline void writeAt(uint8_t * buffer, size_t pos, uint32_t value) {
         buffer[pos] = value & 0xFF;
         buffer[pos+1] = (value >> 8) & 0xFF;
         buffer[pos+2] = (value >> 16) & 0xFF;
         buffer[pos+3] = (value >> 24) & 0xFF;
      }

      uint8_t                  *img_buffer;
      size_t                    img_bufferSize;
      uint32_t                  img_width;
      uint32_t                  img_height;
      bool                      data_owner;
      size_t                    img_rgbSize;
      size_t                    img_rawHeaderBufferSize;
      uint32_t                  img_rawHeaderDataSize;
      struct timeval            img_creationTime;
      struct timeval            img_recordTime;
      MxPEG_ImageMode			img_mode;
   };

}}

#endif /* MXPEG_SRC_MXPEG_IMAGE_HPP_ */
